import static com.kms.katalon.core.testcase.TestCaseFactory.findTestCase

import java.nio.file.Files
import java.nio.file.Path
import java.nio.file.Paths
import java.util.regex.Pattern
import java.util.regex.PatternSyntaxException

import org.apache.poi.ss.usermodel.Sheet
import org.apache.poi.ss.usermodel.Workbook
import org.apache.poi.xssf.usermodel.XSSFWorkbook

import com.kazurayam.materials.MaterialRepository
import com.kazurayam.visualtesting.ManagedGlobalVariable as MGV
import com.kms.katalon.core.configuration.RunConfiguration
import com.kms.katalon.core.webui.keyword.WebUiBuiltInKeywords as WebUI

import internal.GlobalVariable as GlobalVariable
import jp.quick.visualtesting.homepage.autconfig.HpAUTBook
import jp.quick.visualtesting.homepage.autconfig.HpAUTConfig
import jp.quick.visualtesting.homepage.autconfig.HpAUTConfigFormatter
import jp.quick.visualtesting.homepage.autconfig.HpAUTSheet
import jp.quick.visualtesting.homepage.autconfig.HpEnvironment


/**
 * Test Cases/HP/processHpAUTConfig
 * ----
 * GlobalVariable.AUTConfigによって指定されたExcelファイルを読み込んで
 * テスト対象とすべきホームページサービスのシート群を読み込む。各シートにはURL群が記述してあるはず。
 * 
 * URLひとつひとつをWebブラウザで開き、表示完了するまでwaitし、スクリーンショットを取得してPNG画像ファイルとして保存する。
 * URLを問い合わせしてから表示が完了するまでの秒数を計測する。URLが間違っているとかwaitForTextをみつけることができなくて
 * タイムアウトしたことなど、ログをExcelファイルに出力する。
 * 
 * 読み込んだExcelファイルにたとえば1000個のURLが書いてあったとして、そのなかからどのシートつまりサービスを対象とするかを選んだり、
 * URLごとに指定したcategory値によって選択することによって、1000個ではなく20個だけを対象とすることができる。
 * 選択の基準となるパラメータをGlobalVariableから読み取る。テストする人は自分の選択基準を表すExcecution Profileを
 * 作成して、それを使うようなTest Suite Collectionを作るべし。
 */
WebUI.comment("*** GlobalVariable[${MGV.CURRENT_TESTSUITE_ID.getName()}]=${GlobalVariable[MGV.CURRENT_TESTSUITE_ID.getName()]}")
WebUI.comment("*** GlobalVariable[${MGV.CURRENT_TESTSUITE_TIMESTAMP.getName()}]=${GlobalVariable[MGV.CURRENT_TESTSUITE_TIMESTAMP.getName()]}")

if (RunConfiguration.getExecutionProfile() == "default") {
	throw new IllegalArgumentException("Execution Profile \"default\" が選択されましたがdefaultプロファイルは適切でありません。HP_で始まる名前の適切なProfileを選択してください。")
}

if (GlobalVariable.AUTConfig == null) {
	throw new IllegalArgumentException("GlobalVariable.AUTConfigが未定義です。適切なProfileを選択してください。")
}

if (GlobalVariable.ENVIRONMENT == null) {
	throw new IllegalArgumentException("GlobalVariable.ENVIRONMENTが未定義です。適切なProfileを選択してください。")
}

if (GlobalVariable.CATEGORY == null) {
	throw new IllegalArgumentException("GlobalVariable.CATEGORYが未定義です。適切なProfileを選択してください。")
}

// waitForTextとcategoryを入力済みのExcleファイル
File inputExcel = new File(GlobalVariable.AUTConfig)
assert inputExcel.exists()

// 指定されたExcleファイルを入力としてHpAUTConfigオブジェクトをインスタンス化する。
HpAUTConfigFormatter formatter = new HpAUTConfigFormatter()
HpAUTConfig sourceAUTConfig = formatter.parse(new XSSFWorkbook(new FileInputStream(inputExcel)))

String beforeSqueezing = sourceAUTConfig.status()
WebUI.comment("Before squeezing:\n" + beforeSqueezing)

// テスト対象とすべきURL行を絞り込む。指定されたExecution ProfileのGlobalVariableから選択条件を読み込む。
// GlobalVariableが未定義ならばデフォルトを設定する。
List<String> includedSheets = GlobalVariable.INCLUDE_SHEETS ?: []
String env                  = GlobalVariable.ENVIRONMENT    ?: HpEnvironment.本番環境.name()
String category             = GlobalVariable.CATEGORY       ?: '0'
List<Pattern> includeURLs   = compilePatterns(GlobalVariable.INCLUDE_URLS ?: [])

// テストの対象とすべきURLを絞り込んだHpAUTConfigオブジェクトを生成する
HpAUTConfig targetAUTConfig= formatter.squeeze(sourceAUTConfig,
	includedSheets, env, category, includeURLs)

// テスト対象とすべきURLがどれだけの数あるかがわかる情報をMarkdownフォーマットのテキストで取得
String afterSqueezing = sourceAUTConfig.status()
WebUI.comment("After squeezing:\n" + beforeSqueezing)

// Webブラウザを開く
WebUI.openBrowser('')
WebUI.setViewPortSize(1200, 720)

// 処理の結果を記録するためのBook
HpAUTBook resultBook = new HpAUTBook()
// targetAUTConfigに含まれたHpAUTSheetをぜんぶ処理する
for(sheet in targetAUTConfig.getHpAUTBook().getAUTSheetListSorted()) {
	HpAUTSheet result = WebUI.callTestCase(findTestCase("HP/processHpAUTSheet"), ["sheet": sheet])
	resultBook.add(result)
}

// Webブラウザを閉じる
WebUI.closeBrowser()


// URLをブラウザで開いたときに何秒かかったかの記録がresultBookに書き込まれる。
// その記録は有用だからファイルに書き出す。Materialsディレクトリの下にExcelファイルとして書き出す。
MaterialRepository mr = (MaterialRepository)GlobalVariable[MGV.MATERIAL_REPOSITORY.getName()]
assert mr != null
Path outputExcel = mr.resolveMaterialPath(
						GlobalVariable[MGV.CURRENT_TESTCASE_ID.getName()],
						"LOG_" + mr.currentTestSuiteTimestamp + "."
						+ GlobalVariable.ENVIRONMENT + "." 
						+ inputExcel.toPath().getFileName().toString())
new HpAUTConfig(resultBook).writeAsWorkbook(outputExcel)
WebUI.comment("wrote Excel workbook into ${outputExcel.toString()}")

// AfterSqueezeも有用だからファイルに書き出す。Materialsディレクトリの下に.mdファイルとして書き出す。
Path afterSqueeze = mr.resolveMaterialPath(
						GlobalVariable[MGV.CURRENT_TESTCASE_ID.getName()],
						"AUTConfig_status.md")
afterSqueeze.toFile().write(afterSqueeze, "utf-8")

/**
 *
 */
List<Pattern> compilePatterns(List<String> expressions) {
	List<Pattern> patterns = new ArrayList<Pattern>()
	for (regex in expressions) {
		try {
			Pattern ptn = Pattern.compile(regex, Pattern.CASE_INSENSITIVE)
			patterns.add(ptn)
		} catch (PatternSyntaxException e) {
			throw new IllegalArgumentException(e)
		}
	}
	return patterns
}